﻿// ContentLoader Class v4
//
// released under MIT License (X11)
// http://www.opensource.org/licenses/mit-license.php
//
// Author: Niko Helle
// http://www.nikohelle.net

/*
Copyright (c) 2010 Niko Helle

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

package com.nhe.utils{
	import flash.display.Loader;
	import flash.display.Sprite;
	import flash.events.*;
	import flash.net.URLLoader;
	import flash.net.URLLoaderDataFormat;
	import flash.net.URLRequest;
	import flash.net.URLRequestMethod;
	import flash.net.URLVariables;
	import flash.system.ApplicationDomain;
	import flash.system.LoaderContext;
	import flash.utils.ByteArray;

	/**
	 * The ContentLoader class can send & load any kind of data (images,text,variables, bytes)
	 * It simplifies error catching by dispatching only one type of errors (ErrorEvent.ERROR) and storing the source to the errorType property
	 * <p>Events:
	 * Event.COMPLETE is dispatched when send / load is complete<br/>
	 * Event.ERROR is dispatched when an error occures cannot be loaded. The type of error is stored in the errorType property<br/>
	 * ProgressEvent.PROGRESS is dispatched while loading data
	 * Event.INIT is dispatched after the loaded content has initialized
	 * HTTPStatusEvent.HTTP_STATUS is dispatched when HTTP_STATUS changes
	 * </p>
	 * <p>Usage:
	 * <code>
	 * //listeners
	 * _loader = new ContentLoader();
	 * _loader.addEventListener(Event.COMPLETE, loader_INIT);
	 * _loader.addEventListener(ErrorEvent.ERROR, loader_ERROR);
	 * _loader.addEventListener(ProgressEvent.PROGRESS,loader_PROGRESS);
	 * _loader.addEventListener(Event.INIT, loader_INIT);
	 * _loader.addEventListener(HTTPStatusEvent.HTTP_STATUS, loader_HTTP_STATUS);
	 * 
	 * 
	 * //load different media (there are more)
	 * _loader.loadText("variables.txt");
	 * _loader.loadVariables("login.php",{name:"user",pwd:"kfj3i92dlakls"});
	 * _loader.load("image.jpg");
	 * 
	 * // finished
	 * //if it was an image
	 * addChild(_loader.content);
	 * //if it was text
	 * var text = _loader.textData;
	 * 
	 * //if it was variables
	 * var var = _loader.textData.variable; 
	 * 
	 * // clear the loader
	 * _loader.destroy();
	 * _loader = null;
	 * </code>
	 * </p>
	 */

	public class ContentLoader extends Sprite{
	
		
		private var _loader:Loader = null;
		private var _URLloader:URLLoader = null;
		
		private var _content:*;
		//private var _textData:*;
		
		public var loading:Boolean = false;
		private var _loadText:Boolean = false;

		private var _errorType:String;
	
		public function ContentLoader () {
			
				_loader = new Loader();
				addChild(_loader);
				initLoader(_loader.contentLoaderInfo);
			
		}
		
		/**
		 * send / load from/to an url
		 * <code>_loader.load("my.php",{name:"user",pwd:"kfj3i92dlakls"});</code>
		 * 
		 * @param url the url / path to the resource
		 * @param data the data to send in the request
		 * @param method URLRequestMethod.POST (default) or URLRequestMethod.GET
		 * @param keepContext keep the context / ApplicationDomain of the file. Recommended, if the loaded file is a SWF whose library will be used
		 * 
		 * @return
		 *
		 */

		public function load (url:String,data:*=null,method:String="",keepContext:Boolean=false):void {

			loadRequest(buildURLRequest(url,data,method),keepContext);

		}
		
		/**
		 * send / load data according to given URLRequest. Same as load, but the URLRequest is already built
		 * <code>_loader.loadRequest(req,true);</code>
		 * 
		 * @param request the URLRequest
		 * @param keepContext keep the context / ApplicationDomain of the file. Recommended, if the loaded file is a SWF whose library will be used
		 * 
		 * @return
		 *
		 */
				
		public function loadRequest (request:URLRequest, keepContext:Boolean = false) {
			
			_content = null;

		   
			var context:LoaderContext = null;
			context = new LoaderContext();
			context.checkPolicyFile = true;
			if(keepContext == true) {
				context = new LoaderContext();
				context.applicationDomain = new ApplicationDomain();
			}
			loading = true;
			_loadText = false;
			_loader.load(request,context);

			
			

		}
		
		/**
		 * send & load binary data from an url
		 * <code>_loader.loadBinary("my.php",{name:"user",pwd:"kfj3i92dlakls"});</code>
		 * 
		 * @param url the url / path to the resource
		 * @param data the data to send in the request
		 * @param method URLRequestMethod.POST (default) or URLRequestMethod.GET
		 *  
		 * @return
		 *
		 */
		
		public function loadBinary (url:String,data:Object=null,method:String="") {
			
			loadTextRequest(buildURLRequest(url,data,method),URLLoaderDataFormat.BINARY);
		}
		
		/**
		 * send & load text  from an url
		 * <code>_loader.loadText("my.php",{name:"user",pwd:"kfj3i92dlakls"});</code>
		 * 
		 * @param url the url / path to the resource
		 * @param data the data to send in the request
		 * @param method URLRequestMethod.POST (default) or URLRequestMethod.GET
		 *  
		 * @return
		 *
		 */
				
		public function loadText (url:String,data:Object=null,method:String="") {
			
			loadTextRequest(buildURLRequest(url,data,method));

		}
		
		/**
		 * send & load text according to to given URLRequest
		 * <code>_loader.loadTextRequest(request);</code>
		 * 
		 * @param request the URLRequest
		 * @param dataFormat of the loaded data (URLLoaderDataFormat.TEXT / URLLoaderDataFormat.VARIABLES / / URLLoaderDataFormat.BINARY)
		 * 
		 * @return
		 *
		 */
		
		public function loadTextRequest(req:URLRequest,dataFormat:String="") {
			
			if (!_URLloader){
				_URLloader = new URLLoader();
				initLoader(_URLloader);
			}
			if(dataFormat == "") dataFormat = URLLoaderDataFormat.TEXT; 
			_URLloader.dataFormat = dataFormat;
			loading = true;
			_content = null;
			_loadText = true;
			_URLloader.load(req);
		}
		
		/**
		 * send & load variables  from an url
		 * <code>_loader.loadVariables("my.php",{name:"user",pwd:"kfj3i92dlakls"});</code>
		 * 
		 * @param url the url / path to the resource
		 * @param data the data to send in the request
		 * @param method URLRequestMethod.POST (default) or URLRequestMethod.GET
		 *  
		 * @return
		 *
		 */
		
		public function loadVariables(url:String,data:Object=null,method:String="") {
			loadTextRequest(buildURLRequest(url,data,method), URLLoaderDataFormat.VARIABLES);
		}

		/**
		 * send & load text according to to given URLRequest
		 * <code>_loader.loadVariablesRequest(request);</code>
		 * 
		 * @param request the URLRequest
		 * @param keepContext keep the context / ApplicationDomain of the file. Recommended, if the loaded file is a SWF whose library will be used
		 * 
		 * @return
		 *
		 */
		
		public function loadVariablesRequest(req:URLRequest) {
			loadTextRequest(req, URLLoaderDataFormat.VARIABLES);
		}
		
		
		/**
		 * sends bytes to an url and parses the response as variables.
		 * <code>_loader.sendBytes("my.php",bytes,{name:"user",pwd:"kfj3i92dlakls"});</code>
		 * 
		 * @param url the url / path to the resource
		 * @param bytes bytes to send. This is send in the data property of the URLRequest
		 * @param parameters parameters to send in the request. Parameters and values are added to url, because byte data uses the data property
		 *  
		 * @return nothing. The server may return variables, because it will parse the returned data.
		 *
		 */
		
		public function sendBytes (url:String, bytes:ByteArray,parameters:Object= null) {
			

			
			if (parameters != null) {
				url += "?";
				for (var p in parameters) {
					url += p + "=" + escape(parameters[p])+"&";
				}
			}
		
			
			var request:URLRequest = buildURLRequest(url)
			request.contentType = "application/octet-stream";
			request.method = URLRequestMethod.POST;
			request.data = bytes;
			loadVariablesRequest(request);
			
		}
		
		/**
		 * loads (meaning parses) byte data into an object. Commonly used for converting byte image back to an image. Supports SWF, GIF, JPEG, or PNG
		 * <code>_loader.loadBytes(byteArray);</code>
		 * 
		 * @param bytes byte data
		 * @param bytes bytes to send. This is send in the data property of the URLRequest
		 *  
		 * @return
		 *
		 */
		
		public function loadBytes(bytes:ByteArray) {
			_content = null;
			_loadText = false;
			_loader.loadBytes(bytes);
		}
		
		/**
		 * creates an URLRequest from given parameters
		 * <code>_loader.buildURLRequest("my.php",{name:"user",pwd:"kfj3i92dlakls"});</code>
		 * 
		 * @param url the url / path to the resource
		 * @param data the data to add to the request
		 * @param method URLRequestMethod.POST (default) or URLRequestMethod.GET
		 *   
		 * @return an URLRequest
		 *
		 */
		
		public function buildURLRequest (url:String,data:Object=null,method:String=""):URLRequest {
			

			var req:URLRequest = new URLRequest(url); 
			
			if (data != null) {
				var vars:URLVariables = new URLVariables(); 
				if (method != URLRequestMethod.GET) method = URLRequestMethod.POST
				req.method = method;
				
				req.data = data;
				
			}
			
			return req;
			
		}
		
		private function initLoader(loader:IEventDispatcher) {
			loader.addEventListener(Event.COMPLETE, loader_COMPLETE);
			loader.addEventListener(Event.INIT, loader_INIT);
			loader.addEventListener(HTTPStatusEvent.HTTP_STATUS, loader_HTTP_STATUS);
			loader.addEventListener(IOErrorEvent.IO_ERROR, loader_IO_ERROR);
			loader.addEventListener(ProgressEvent.PROGRESS, loader_PROGRESS);
			loader.addEventListener(SecurityErrorEvent.SECURITY_ERROR, loader_SECURITY_ERROR);
		}
		
		private function formatLoader(loader:IEventDispatcher) {
			loader.removeEventListener(Event.COMPLETE, loader_COMPLETE);
			loader.removeEventListener(Event.INIT, loader_INIT);
			loader.removeEventListener(HTTPStatusEvent.HTTP_STATUS, loader_HTTP_STATUS);
			loader.removeEventListener(IOErrorEvent.IO_ERROR, loader_IO_ERROR);
			loader.removeEventListener(ProgressEvent.PROGRESS, loader_PROGRESS);
			loader.removeEventListener(SecurityErrorEvent.SECURITY_ERROR, loader_SECURITY_ERROR);
		}


		private function loader_COMPLETE(e:Event) {
			loading = false;
			try{
				_content = _loader.content
				if (_URLloader && _loadText) _content = _URLloader.data;
			}
			catch (e:Error) {
				_errorType = ErrorEvent.ERROR;
				dispatchEvent(new ErrorEvent(ErrorEvent.ERROR));
				return;
				
			}
	
			dispatchEvent(e);
		}
		
		private function loader_INIT(e:Event) {
			//complete is dispatched before this so no need to set _content again
			
			dispatchEvent(e);
		}
		
		private function loader_HTTP_STATUS(e:HTTPStatusEvent) {

			dispatchEvent(e);
		}
		
		private function loader_IO_ERROR(e:IOErrorEvent) {
			_errorType = IOErrorEvent.IO_ERROR;
			loading = false;
			dispatchEvent(new ErrorEvent(ErrorEvent.ERROR));
		}
		
		private function loader_PROGRESS(e:ProgressEvent) {
			dispatchEvent(e);
		}
		
		private function loader_SECURITY_ERROR(e:SecurityErrorEvent) {
			_errorType = SecurityErrorEvent.SECURITY_ERROR;
			loading = false;
			dispatchEvent(new ErrorEvent(ErrorEvent.ERROR));
		}
		
		/**
		 * forces the loading process to end
		 * <code>_loader.cancel();</code>
		 */
		
		public function cancel() {

			if (!_URLloader) {
				_loader.close();
				_loader.unload();
			}
			else {
				_URLloader.close();

			}
			
			try {
				if (!_URLloader) _loader.load(null);
				else _URLloader.load(null);
			}
			catch (e:Error) {
				//triggered always
			}
			loading = false;
			
		}
		
		/**
		 * destroys the this loader and stored data
		 * <code>_loader.destroy();</code>
		 */
		
		public function destroy() {

			if (!_loader) return;
			_loader.unload();
			try {
				_loader.load(null);
			}
			catch (e:Error) {
				//triggered always
			}
			formatLoader(_loader.contentLoaderInfo);
			if (_URLloader) {
				formatLoader(_URLloader);
				try {
					_URLloader.load(null);
				}
				catch (e:Error) {
					//triggered always
				}
			}
			removeChild(_loader);
			_URLloader = null;
			_loader = null;
			_content = null;
		
			loading = false;

		}
		
		/**
		 * get the Loader instance
		 * <code>var loader = _loader.loader</code>
		 *  
		 * @return the used loader
		 *
		 */
		
		public function get loader():Loader {
			return _loader;
		}
		
		/**
		 * get the loaded content
		 * <code>var content = _loader.content</code>
		 *  
		 * @return the content
		 *
		 */
		
		public function get content():* { return _content; }
		
		/**
		 * get the type of occured error
		 * <code>var errorType = _loader.errorType</code>
		 *  
		 * @return the type of the error (IOErrorEvent.IO_ERROR or SecurityErrorEvent.SECURITY_ERROR or _errorType = ErrorEvent.ERROR;)
		 *
		 */
		
		public function get errorType():String { return _errorType; }

		
		
	
		
}
}